/* 
   The Jitterbug report tracking system

   Copyright (C) Andrew Tridgell 1997

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
  process a new piece of mail coming into the system. The mail is on stdin.
  The default directory to place the mail is given as the only argument to the
  program. If this is not specified then "incoming" is assumed.
 */

#include "jitterbug.h"

int done_chroot = 0;
int guest = 0;
char *user="";


void fatal(char *why)
{
	fprintf(stderr,"%s\n", why);
	exit(1);
}

/* get the message id of a email if possible */
static int getid(char *fname)
{
	char *buf, *p1, *p2, *p3;
	struct stat st;
	int fd;

	if (stat(fname, &st)) return 0;

	buf = malloc(st.st_size+1);
	if (!buf) return 0;
	
	fd = open(fname,O_RDONLY);
	if (fd == -1) return 0;

	if (read(fd, buf, st.st_size) != st.st_size) return 0;

	buf[st.st_size] = 0;

	p1 = buf;
	while ((p1 = strstr(p1, "\nSubject:"))) {
		p1 += 9;
		p2 = strchr(p1, '\n');
		if (!p2) return 0;
		*p2++ = 0;
		p3 = p1;
		while ((p3 = strstr(p3, "PR#"))) {
			if (atoi(p3+3)) return atoi(p3+3);
			p3 += 3;
		}
		p3 = p1;
		while ((p3 = strstr(p3, "BUG#"))) {
			if (atoi(p3+4)) return atoi(p3+4);
			p3 += 4;
		}
		p1 = p2;
	}
	return 0;
}


int process_mail(char *def_dir)
{
	FILE *f = fopen(".newmsg","w");
	char buf[100], fname[1000];
	int id, followup=0;
	char idname[20];
	char *p, *dir, *basename=NULL;

	/* read the message into a temporary file */
	while (!feof(stdin)) {
		int len = fread(buf, 1, sizeof(buf), stdin);
		if (len > 0)
			fwrite(buf, 1, len, f);
	}
	fclose(f);

	fname[0] = 0;

	/* work out if it has an existing id */
	id = getid(".newmsg");
	sprintf(idname,"%d", id);

	if (id != 0) {
		basename = find_file(".", idname);
		if (basename) {
			followup = count_followups(basename, NULL)+1;
			sprintf(fname,"%s.followup.%d", basename, followup);
		}
	}

	if (! *fname) {
		char *idfile = load_file(".nextid", NULL);
		int nextid=1;
		if (idfile) nextid = atoi(idfile);
		sprintf(idname,"%d", nextid+1);
		save_file(".nextid", idname);
		sprintf(idname,"%d", nextid);
		sprintf(fname,"%s/%d", def_dir, nextid);
	}

	p = strrchr(fname,'/');
	if (!p) {
		fprintf(stderr,"invalid filename\n");
		exit(1);
	}
	*p = 0;
	dir = strdup(fname);
	*p = '/';

	if (!is_directory(dir)) {
		mkdir(dir,0755);
	}

	if (rename(".newmsg", fname)) {
		fprintf(stderr,"rename .newmsg %s failed\n", fname);
	}
	
	chdir(dir);
	notify_dir(dir, idname, "new message %s\n", fname);

	if (followup) {
		notify_msg(idname,"new followup %d\n", followup);
	}

	return 0;
}


int main(int argc, char *argv[])
{
	int ret;
	char *def_dir = "incoming";

	if (chdir(BASEDIR)) {
		fprintf(stderr,"Failed to chdir to %s\n", BASEDIR);
		exit(1);
	}

	if (argc > 1)
		def_dir = argv[1];

	umask(022);

	lock_system();
	ret = process_mail(def_dir);
	unlock_system();

	return ret;
}
